package cz.aimtec.dci.coreGate.struts.coreServices;

import cz.aimtec.alfa.AlfaObject;
import cz.aimtec.alfa.AlfaWorkspace;
import cz.aimtec.dci.DciApplicationServlet;
import cz.aimtec.dci.DciApplicationWorkspace;
import cz.aimtec.dci.DciCompanyWorkspace;
import cz.aimtec.dci.TCPIPMessage.LogValue;
import cz.aimtec.dci.TCPIPMessage.WatchdogLog;
import cz.aimtec.dci.TCPIPMessage.WatchdogLogCollection;
import cz.aimtec.dci.action.DciAction;
import cz.aimtec.dci.action.DciActionType;
import cz.aimtec.dci.action.DciForwardKey;
import cz.aimtec.dci.coreservices.api.model.TransactionFieldValue;
import cz.aimtec.dci.microservices.ServiceList;
import cz.aimtec.dci.services.Service;
import cz.aimtec.dci.services.SystemServicesDispatcher;
import cz.aimtec.dci.websocket.server.WebSocketServer;
import cz.aimtec.dci.websocket.server.WebSocketType;
import cz.aimtec.util.Convert;
import cz.aimtec.util.exception.ApplicationRuntimeException;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Core services action.
 * Satisfies core services controller requests of core gate. 
 *
 * @author	hors
 * @version	7. 11. 2017/hors - created<br>
 */
public class CoreServicesAction extends DciAction {

	@Override
	public ActionForward execute(ActionMapping pMapping,
	                             ActionForm pForm,
	                             HttpServletRequest pRequest,
	                             HttpServletResponse pResponse) throws Exception {
		CoreServicesActionForm form = (CoreServicesActionForm) pForm;
		String action = form.getAction();
		DciApplicationWorkspace applicationWorkspace = DciApplicationWorkspace.getInstance();
		SystemServicesDispatcher applicationDispatcher = ((DciApplicationServlet) applicationWorkspace.getApplicationServlet()).getSystemServicesDispatcher();
		SystemServicesDispatcher executionDispatcher;
		// ------------------------ looking for services dispatcher
		executionDispatcher = AlfaObject.isIdValid(form.getCompanyId())
				? applicationWorkspace.getCompanyWorkspace(form.getCompanyId()).getSystemServicesDispatcher()
						: applicationDispatcher;
				
		// ------------------------- action handling
		if (DciActionType.CONFIGURATION.value().equals(action)) {
			JSONArray response = new JSONArray();

			// ------------------------ application services
			addServicesToResponse(response, applicationDispatcher);
			
			// ------------------------ company services
			for (AlfaWorkspace workspace : applicationWorkspace.getSubWorkspaces().values()) {
				DciCompanyWorkspace companyWorkspace = (DciCompanyWorkspace) workspace;
				addServicesToResponse(response, companyWorkspace.getSystemServicesDispatcher());
			}
			
			// ------------------------ save the result
			form.setSerializedResponse(response.toString());
			
		} else if (DciActionType.EXECUTE.value().equals(action)) {
			// ------------------------ execute the service or event
			if (Convert.isBlank(form.getEventCode())) {
				executionDispatcher.runService(form.getServiceCode());
			} else {
				executionDispatcher.runEvent(form.getServiceCode(), form.getEventCode());
			}
			
		} else if (DciActionType.REGISTER_LICENSE.value().equals(action)) {
			// --------------------- service event license registration
			executionDispatcher.registerEventLicense(form.getServiceCode(), form.getEventCode());
			
		} else if (DciActionType.DEREGISTER_LICENSE.value().equals(action)) {
			// --------------------- service event license deregistration
			executionDispatcher.deregisterEventLicense(form.getServiceCode(), form.getEventCode());
			
		} else if (DciActionType.HANDLE_MESSAGE.value().equals(action)) {
			// --------------------- incoming message handling
            List<LogValue> logValues = new ArrayList<>();
            List<TransactionFieldValue> initialParameters = new ArrayList<>();
			
			if (Convert.isNotBlank(form.getLogValues())) {
                LogValue[] logValuesNative =
						ServiceList.coreServices.getJsonConverter().readValue(form.getLogValues(), LogValue[].class);
                logValues = Arrays.asList(logValuesNative);
			}
			
			if (Convert.isNotBlank(form.getInitialParameters())) {
                TransactionFieldValue[] initialParametersNative
						= ServiceList.coreServices.getJsonConverter().readValue(form.getInitialParameters(),
																				TransactionFieldValue[].class);
                initialParameters = Arrays.asList(initialParametersNative);
			}

			executionDispatcher.handleEventMessage(form.getServiceCode(),
			                                       form.getEventCode(),
			                                       form.getListenerCode(),
			                                       form.getMessageType(),
			                                       form.getMessageBytesNative(),
			                                       form.getDataVector(),
			                                       form.getCommandResponse(),
			                                       form.getMachineState(),
			                                       form.getMachineCode(),
			                                       form.getDescription(),
			                                       form.getCreateDateNative(),
                                                   form.getUniqueIdentifier(),
			                                       form.getOpcNodeId(),
			                                       form.getOpcAttributeTypeId(),
			                                       form.getTransactionDefinitionName(),
			                                       logValues,
			                                       initialParameters);
			
		} else if (DciActionType.ARCHIVE_MESSAGES.value().equals(action)) {

			try (Connection connection = applicationWorkspace.getNewStandardConnection(getClass())) {
				// --------------------- incoming messages archiving
				executionDispatcher.archiveEventMessages(form.getArchivedIdentifersNative(), connection);
			} catch (SQLException e) {
				log.error("Couldnt get connection for archiving incomming messages with error {}", e);
				throw new ApplicationRuntimeException("Couldnt get connection for archiving incomming messages", e);
			}

			
		} else if (DciActionType.SAVE_WATCHDOG_LOGS.value().equals(action)) {
			// --------------------- watchdog logs saving
			WatchdogLogCollection watchdogLogs = new WatchdogLogCollection(applicationWorkspace);
			watchdogLogs.parseJsonArray(WatchdogLog.class, new JSONArray(form.getSerializedRequestBody()));

			try (Connection connection = applicationWorkspace.getNewStandardConnection(getClass())) {
				executionDispatcher.saveWatchdogLogs(form.getServiceCode(), watchdogLogs, connection);
			} catch (SQLException e) {
				log.error("Couldnt get connection for saving watchdog logs with error {}", e);
				throw new ApplicationRuntimeException("Couldnt get connection for saving watchdog logs", e);
			}


		} else if (DciActionType.SAVE_RFC_SERVER_STATE.value().equals(action)) {
			// --------------------- SAP RFC server state saving
			JSONObject jsonServerState = new JSONObject(form.getSerializedRequestBody());
			try (Connection connection = applicationWorkspace.getNewStandardConnection(getClass())) {
				executionDispatcher.saveRfcServerState(form.getServiceCode(),
						jsonServerState.getString("state"),
						jsonServerState.getInt("connectionCount"),
						connection);
			} catch (SQLException e) {
				log.error("Couldnt get connection for SAP RFC server state saving with error {}", e);
				throw new ApplicationRuntimeException("Couldnt get connection for SAP RFC server state saving", e);
			}

			
		} else if (DciActionType.CHECK_RFC_TID.value().equals(action)) {
			// ------------------------ SAP RFC TID check
			JSONObject response = new JSONObject();
			response.put("status", executionDispatcher.checkRfcTid(form.getServiceCode()));
			form.setSerializedResponse(response.toString());
			
		} else if (DciActionType.HANDLE_RFC_MESSAGE.value().equals(action)) {
			JSONObject jsonMessage = new JSONObject(form.getSerializedRequestBody());
			executionDispatcher.handleRfcMessage(form.getServiceCode(),
			                                     jsonMessage.getString("processingTransactionCode"),
			                                     jsonMessage.getString("document"),
			                                     jsonMessage.getString("identificationNumber"));
			
		} else if (DciActionType.SAVE_RFC_SERVER_ERROR.value().equals(action)) {
			JSONObject jsonMessage = new JSONObject(form.getSerializedRequestBody());
			executionDispatcher.saveRfcServerError(form.getServiceCode(),
			                                       jsonMessage.getString("errorMessage"),
			                                       jsonMessage.getString("identificationNumber"));
			
		} else if (DciActionType.SAVE_MACHINE_RESPONSE_DURATION.value().equals(action)) {
			JSONObject jsonResponseDuration = new JSONObject(form.getSerializedRequestBody());
			executionDispatcher.saveMachineResponseDuration(form.getServiceCode(),
			                                                form.getEventCode(),
			                                                jsonResponseDuration.getString("opcNodeId"),
			                                                jsonResponseDuration.optString("machineCode"),
			                                                jsonResponseDuration.getString("changedValue"),
			                                                jsonResponseDuration.getString("author"));
		} else if (DciActionType.SERVICES_MONITOR.value().equals(action)) {
			String result = pRequest.getReader().lines().collect(Collectors.joining());
			Collection<List<WebSocketServer>> servers = WebSocketType.SERVICES.chooseSourceMap().values();
			for (List<WebSocketServer> server : servers) {
				for (WebSocketServer realServer : server){
					realServer.sendMessage(result);
				}
			}
		}
		
		// ---------------------- return
		return findForward(pMapping, DciForwardKey.SUCCESS);
	}
	
    private void addServicesToResponse(JSONArray pResponse,
                                       SystemServicesDispatcher pServicesDispatcher) throws JSONException {
		JSONObject responseItem;
		
		for (Service service : pServicesDispatcher.getTaskList()) {
			if (!service.isInstanceDependent()) {
				responseItem = (JSONObject) service.toJsonObject();
				pResponse.put(responseItem);
			}
		}
	}
}
