package cz.aimtec.dci.action;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import cz.aimtec.dci.action.DciCommonActionStrategy.ActionName;
import cz.aimtec.util.exception.ApplicationRuntimeException;

/**
 * Enumerator for common action names that are used as an action parameter value.
 * 
 * @author: asel
 */
public enum DciActionType {

	// Common actions
	PREPARE("Prepare"),
	CREATE("Create"),
	EDIT("Edit"),
	UPDATE("Update"),
	RESET("Reset"),
	SEARCH("Search"),
	VIEW("View"),
	SAVE("Save"),
	SAVECLOSE("SaveClose"),
	REFRESH("Refresh"),
	RELOAD("reload"),
	DELETE("Delete"),
	CLOSE("Close"),
	CANCEL("Cancel"),
	DISABLE("Disable"),
	FINISH("Finish"),
	OPEN("Open"),
	OPEN_EDIT("OpenEdit"),
	
	// Navigation actions
	FORWARD("Forward"),
	BACKWARD("Backward"),
	BACK("Back"),
	CONTINUE("Continue"),
	DEBUG_CONTINUE("debugContinue"),
	DEBUG_CLOSE("debugClose"),
	RUN_ACTION("RunAction"),
	RUN_ACTION_DEBUG("RunActionDebug"),
	
	// All/Selected actions
	PROCESS_ALL("ProcessAll"),
	PROCESS_SELECTED("ProcessSelected"),
	PROCESS_DEBUG_SELECTED("ProcessDebugSelected"),
	ASSIGN_ALL("AssignAll"),
	ASSIGN_SELECTED("AssignSelected"),
	DELETE_SELECTED("DeleteSelected"),
	
	// Print actions
	PRINT("Print"),
	PRINT_SELECTED("PrintSelected"),
	
	// Notes actions
	ADD_NOTE("AddNote"),
	DELETE_NOTE("DeleteNote"),
	
	// Copy actions
	CREATE_COPY("CreateCopy"),
	CREATE_BATCH("CreateBatch"),
	CREATE_AS_COPY("CreateAsCopy"),
	AFTER_CREATE_BATCH("AfterCreateBatch"),
	
	// coreGate core services actions
	CONFIGURATION("configuration"),
	EXECUTE("execute"),
	REGISTER_LICENSE("registerLicense"),
	DEREGISTER_LICENSE("deregisterLicense"),
	HANDLE_MESSAGE("handleMessage"),
	ARCHIVE_MESSAGES("archiveMessages"),
	SAVE_WATCHDOG_LOGS("saveWatchdogLogs"),
	INITIALIZE_MACHINE("initializeMachine"),
	SAVE_RFC_SERVER_STATE("saveRfcServerState"),
	CHECK_RFC_TID("checkRfcTid"),
	HANDLE_RFC_MESSAGE("handleRfcMessage"),
	SAVE_RFC_SERVER_ERROR("saveRfcServerError"),
	SAVE_MACHINE_RESPONSE_DURATION("saveMachineResponseDuration"),
	SERVICES_MONITOR("servicesMonitor"),
	
	// coreGate data integrator actions
	EXPORT_CONFIRMATION("ExportConfirmation"),
	
	// Other (commonly used) actions
	RESET_FILTER("ResetFilter"),
	CHANGE_TAB("ChangeTab"),
	APPEND_CHILD("AppendChild"),
	CHANGE_VIEW("ChangeView"),
	DELETE_VIEW("DeleteView"),
	CREATE_LINE("CreateLine"),
	DELETE_LINE("DeleteLine"),
	DELETE_ITEM("DeleteItem"),
	AFTER_TYPING("AfterTyping"),
	AFTER_SCANNING("afterScanning"),
	DEBUG_NEXT_STEP("debugNextStep"),
	DELETE_PARAMETER("DeleteParameter"),
	DEASSIGN_PRODUCT("DeassignProduct"),
	
	// Window actions
	
	ADD_WINDOW("addWindow"),
	REMOVE_WINDOW("removeWindow"),
	MOVE_WINDOW("moveWindow"),
	SAVE_WINDOWS_CHANGES("saveWindowsChanges"),
	CHANGE_COLLECTION_SETTINGS_VISIBILITY("changeEditCollectionSettingsVisibility"),
	CHANGE_MENU_KEY("changeMenuKey"),
	
	// Other (scarcely used) actions
	PREPARE_SEARCH("PrepareSearch"),
	SAVE_DEBUG("SaveDebug"),
	SAVE_ASSIGNEE_COLLECTION("SaveAsigneeCollection"),
	SAVE_AND_RUN_ACTION("SaveActionRunTrn"),
	SAVE_AND_RUN_ACTION_DEBUG("SaveActionDebugTrn"),
	SAVE_AND_RUN_ACTION_CLOSE("SaveActionRunTrnClose"),
	SAVE_AND_RUN_ACTION_DEBUG_CLOSE("SaveActionDebugTrnClose"),
	FINISH_SELECTED("FinishSelected"),
	CHOOSE_COMPANY("ChooseCompany"),
	CHOOSE_TRANSACTION("ChooseTransaction"),
	PAGE_SORT_EXPORT("pageSortExport"),
	CHECK_FAVORITE("checkFavorite"),
	NEW_MESSAGE("NewMessage"),
	SEND("Send"),
	EXECUTE_ATTACHMENT("ExecuteMessageAttachment"),
	EXECUTE_ATTACHMENT_DEBUG("ExecuteMessageAttachmentDebug"),
	ACCEPT("Accept"),
	UPLOAD_FILE("UploadFile"),
	AFTER_SAVE("AfterSave"),
	SELECT_TRANSACTION_DEFINITION ("SelectTransDef"),
	GENERATE("Generate"),
	COMPARE("Compare"),
	BACKUP("backup"),
	RESTORE("restore"),
	FILTER("filter"),
	VALIDATE_PACKAGE("ValidatePackage"),
	IGNORE_WARNINGS("IgnoreWarnings"),
	JUMP_TO_SAVEPOINT("JumpToSavePoint"),
	JUMP("Jump"),
	MASS_COPY("massCopy"),
	MASS_CHANGE("MassChange"),
	IMPORT("Import"),
	EXPORT("Export"),
	CHANGE_PACKAGE_DEFINITION("changePackageDefinition"),
	ADD_CHILD_PACKAGE("addChildPackage"),
	ADD_TOP_PACKAGE("addTopPackage"),
	DELETE_PACKAGE("deletePackage"),
	VALIDATE_AND_CONTINUE("validateAndContinue"),
	RECOUNT("Recount"),
	GO_TO_PAGE("goToPage"),
	REJECT_PACKAGE("RejectPackage"),
	REJECT_PACKAGE_DEBUG("RejectPackageDebug"),	
	REJECT_PACKAGE_ALL("RejectPackageAll"),
	REJECT_PACKAGE_ALL_DEBUG("RejectPackageAllDebug"),
	REJECT_PACKAGE_FOR_LINE("RejectPackageForLine"),
	REJECT_PACKAGE_FOR_LINE_DEBUG("RejectPackageForLineDebug"),
	GENERATE_PACKAGES("GeneratePackages"),
	GENERATE_PACKAGES_DEBUG("GeneratePackagesDebug"),
	GENERATE_PACKAGES_FOR_LINE("GeneratePackagesForLine"),
	GENERATE_PACKAGES_FOR_LINE_DEBUG("GeneratePackagesForLineDebug"),
	DISPLAY_PDF("displayPdf"),
	VIEW_REPORT("viewReport"),
	VIEW_REPORT_IFRAME_ACTION_AND_FORWARD("viewReportIframe"),
	ACTION_REPORT_CHANGED("ReportChanged"),
	ACTION_REPORT_TYPE_CHANGED("ReportTypeChanged"),
	ACTION_LANGUAGE_CHANGED("LanguageChanged"),
	ACTION_SSRS_GENERATOR("SsrsGenerator"),
	ACTION_GENERATE_URL("GenerateUrl"),
	CLEAR_CACHE("ClearCache"),
	AJAX_CLEAR_CACHE("AjaxClearCache"),
	CREATE_NEW_VERSION("CreateNewVersion"),
	AUTHENTICATE_ACCESS_KEY("authenticationAccessKeyAjax"),
	UPLOAD_FILE_FROM_XP_TRANSACTION("uploadFileFromXPTransaction"),
	SHOW_UPLOAD_FILE_FROM_XP_TRANSACTION("showUploadFileFromXPTransaction"),
	AUTOLOGON_ERROR("autologonError"),
	AUTOLOGON_CANCEL("autologonCancel"),
	AUTOLOGON_LOG_OVER("autologonLogOver"),
	AFTER_BEEP("AfterBeep");
	
	//****************************************************************

	// Value of ActionName
	private String value = null;

	/**
	 * Private ActionName constructor.
	 */
	private DciActionType(String value) {
		this.value = value;
	}
	
	/**
	 * Returns appropriate action for the given action name
	 * 
	 * @param value
	 * @return
	 */
	public static DciActionType findAction(String value) {
		DciActionType foundAction = null;
		for (DciActionType action: DciActionType.values()) {
			if (action.value().equalsIgnoreCase(value)) {
				foundAction = action;
			}
		}
		return foundAction;
	}
	
	/**
	 * Invokes method associated with the action type. Method names are constructed as: actionName + "Action"
	 * 
	 * @param actionClass
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return
	 */
	public ActionForward invokeAction(DciAction actionClass, ActionMapping mapping, ActionForm form, HttpServletRequest request,
											HttpServletResponse response) {
	
		try {
			for (Method classMethod: actionClass.getClass().getMethods()) {
				ActionName annotation = classMethod.getAnnotation(ActionName.class);
				if (annotation != null
						&& this.equals(annotation.value()[0])) {
					return (ActionForward) classMethod.invoke(actionClass, mapping, form, request, response);
				}
			}
		} catch (IllegalArgumentException e) {
			throw new ApplicationRuntimeException("Illegal argument error. ", e);
		} catch (IllegalAccessException | InvocationTargetException e) {
			throw new ApplicationRuntimeException("Can't call method. ", e);
		}
		
		return null;
	}

	@Override
	public final String toString() {
		return value;
	}
	
	/**
	 * Returns String value
	 * @return String
	 */
	public final String value() {
		return value;
	}
}
