package cz.aimtec.dci.services.technology.tcpip;

import cz.aimtec.dci.coregate.api.model.coreservices.OPCSubscriptionItemStatus;
import cz.aimtec.util.datatype.DateTime;


/**
 * Subscription item for TCPIPListener subscriptions.
 * In this version it is primarily designed for OPC subscriptions because other TCPIP listeners don't support subscriptions.
 *
 * @author asel
 */
public class SubscriptionItem {
    private Integer subscriptionId;
    private String nodeId;
    private String attributeId;
    private String transactionCode;
    private DateTime lastRunTime;
    private String publishingInterval;
    private boolean subscribed;

    /**
     * Default constructor.
     *
     * @param nodeId
     * @param attributeId
     * @param transactionCode
     */
    public SubscriptionItem(Integer subscriptionId, String nodeId, String attributeId, String transactionCode, DateTime lastRunTime, String publishingInterval, boolean subscribed) {
        this.subscriptionId = subscriptionId;
        this.nodeId = nodeId;
        this.attributeId = attributeId;
        this.transactionCode = transactionCode;
        this.lastRunTime = lastRunTime;
        this.publishingInterval = publishingInterval;
        this.subscribed = subscribed;
    }

    /**
     * Basic constructor.
     *
     * @param pSubscriptionStatus subscription status as template
     */
    public SubscriptionItem(OPCSubscriptionItemStatus pSubscriptionStatus) {
        this(
            pSubscriptionStatus.getSubscriptionId(),
            pSubscriptionStatus.getNodeId(),
            pSubscriptionStatus.getAttributeId(),
            pSubscriptionStatus.getTransactionCode(),
            DateTime.convertToDateTime(pSubscriptionStatus.getLastRunTime()),
            pSubscriptionStatus.getPublishingInterval(),
            pSubscriptionStatus.getSubscribed()
        );
    }

    public String getNodeId() {
        return nodeId;
    }

    public String getNodeIdShort() {
        return (nodeId.length() <= 20)
                ? nodeId
                : nodeId.substring(0, 17) + "...";
    }

    public Integer getSubscriptionId() {
        return subscriptionId;
    }

    public String getAttributeId() {
        return attributeId;
    }

    public String getTransactionCode() {
        return transactionCode;
    }

    public DateTime getLastRunTime() {
        return lastRunTime;
    }

    public String getPublishingInterval() {
        return publishingInterval;
    }

    public String getSubscribed() {
        if (subscribed) {
            return "img/Bullet-Green.png";
        } else {
            return "img/Bullet-Red.png";
        }
    }
}
