package cz.aimtec.dci.websocket.server;

import java.util.List;
import java.util.Map;

import javax.websocket.MessageHandler;
import cz.aimtec.dci.websocket.handlers.DefaultMessageHandler;
import cz.aimtec.dci.websocket.handlers.MessageNotificationHandler;
import cz.aimtec.dci.websocket.handlers.PingHandler;

public enum WebSocketType {
	SEMAPHORE {
		@Override
		public Map<String, List<WebSocketServer>> chooseSourceMap() {
			return WebSocketManager.getConnectionsSemaphoreMap();
		}
		
		@Override
		public MessageHandler.Whole<String> createMessageHandler(WebSocketServer server) {
			return new PingHandler(server);
		}
		
		@Override
		public int getPingInterval() {
			// without waiting
			return 0;
		}
		
	},
	MESSAGES {
		@Override
		public Map<String, List<WebSocketServer>> chooseSourceMap() {
			return WebSocketManager.getConnectionsMessagesMap();
		}
		
		@Override
		public String getConnectionIdentifier(WebSocketServer server) {
			return server.getUserId().toString();
		}
		
		@Override
		public MessageHandler.Whole<String> createMessageHandler(WebSocketServer server) {
			return new MessageNotificationHandler(server);
		}
	},
	TERMINALS {
		@Override
		public Map<String, List<WebSocketServer>> chooseSourceMap() {
			return WebSocketManager.getConnectionsMessagesMap();
		}

		@Override
		public String getConnectionIdentifier(WebSocketServer server) {
			return server.getUniqueCode();
		}

		@Override
		public MessageHandler.Whole<String> createMessageHandler(WebSocketServer server) {
			return new MessageNotificationHandler(server);
		}
	},
	SERVICES {
		@Override
		public Map<String, List<WebSocketServer>> chooseSourceMap() {
			return WebSocketManager.getSystemServicesConnectionMap();
		}
	};
	
	/**
	 * Returns map with appropriate connections for the given type
	 * @return
	 */
	public abstract Map<String, List<WebSocketServer>> chooseSourceMap();
	
	/**
	 * Creates and returns message handler for the given connection
	 * @return
	 */
	public MessageHandler.Whole<String> createMessageHandler(WebSocketServer server) {
		return new DefaultMessageHandler(server.getSession());
	}
	
	/**
	 * Returns value of identifier which is used to identify connection in
	 * connections map for the given type
	 * @return
	 */
	public String getConnectionIdentifier(WebSocketServer server) {
		return server.getUniqueCode();
	}
	
	/**
	 * @param server this parameter is necessary for the override method 
	 */
	public WebSocketPing createPingThread(WebSocketServer server) {
		return new WebSocketPing(server);
	}
	
	public int getPingInterval() {
		// 30secs
		return 30 * 1000;
	}
	
}