/**
 * XP transaction modules.
 */
define(['cz.aimtec.dci.websocket'], function() {

    /**
     * Namespace initialization.
     */
    var utilsNs = cz.aimtec.dci.utils;
    var windowUtils = utilsNs.initializeNamespace('cz.aimtec.dci.window.utils');
    var ns = utilsNs.initializeNamespace('cz.aimtec.dci.systemServices');
    var websocketFramework = utilsNs.initializeNamespace('cz.aimtec.dci.websocket');
    var logger = utilsNs.initializeNamespace('cz.aimtec.dci.logger');

    /**
     * Constants.
     */
    var WEB_SOCKET_STORAGE_KEY = 'SERVICES';
    var WEB_SOCKET_ENDPOINT = 'websocket/' + WEB_SOCKET_STORAGE_KEY + '/';
    var RELEASE_MESSAGE = 'release';
    var PING_MESSAGE = 'ping';
    var PONG_MESSAGE = 'pong';

    var socket;


    /**
     * Initialization of semaphore client
     *
     */
    ns.initialize = function() {
        var userId = 'dci';
        var semaphoreCode = navigator.userAgent;
        var currentWindow = windowUtils.getCurrentWindow();
        socket = openSocket(userId, semaphoreCode);
    };

    function openSocket(userId, semaphoreCode) {
        /*logger.logDebug('{cz.aimtec.dci.transaction.xp.modules.semaphore.js:openSocket,_userId:'
        		+ userId + ',_semaphoreCode:' + semaphoreCode + '}');*/
        console.log(userId + semaphoreCode);
        var semaphoreStorage = $(window);
        var websocketParams = '?code=' + semaphoreCode + '&userid=' + userId;
        var websocketURL = websocketFramework.getWebSocketServerPath(WEB_SOCKET_ENDPOINT) + websocketParams;
        return websocketFramework.openSocket(semaphoreStorage, WEB_SOCKET_STORAGE_KEY, websocketURL, onMessage);
    }

    function updateValue(parentHtmlElement, key, value) {
        var cellElement = parentHtmlElement.querySelector("[attribute=" + key + "]");
        if(cellElement && value) {
            if(cellElement.getAttribute("data-type") == 'indicator') {
                if(value==='running') {
                    cellElement.innerHTML = cellElement.innerHTML.replaceAll(/img\/.*\.png/g, "img/Bullet-Green.png");
                    return;
                }
                if(value==='down') {
                cellElement.innerHTML = cellElement.innerHTML.replaceAll(/img\/.*\.png/g, "img/Bullet-Red.png");
                    return;
                }
                if(value) {
                    cellElement.innerHTML = cellElement.innerHTML.replaceAll(/img\/.*\.png/g, "img/Bullet-Green.png");
                } else {
                    cellElement.innerHTML = cellElement.innerHTML.replaceAll(/img\/.*\.png/g, "img/Bullet-Red.png");
                }
            } else if(cellElement.getAttribute("data-type") == 'date') {
                var stringDate = new Date(value);
                cellElement.textContent = (new Date(value).toLocaleString("cs-CZ")).replaceAll('. ', '.');
              } else {
                   cellElement.textContent = value;
              }
        }
    }

    function updateSubscriptions(htmlElement, propertyValues) {
        var subscriptionProperties = Object.keys(propertyValues);
        subscriptionProperties.forEach(function (key) {
            updateValue(htmlElement, key, propertyValues[key]);
        })
    }

    function updateEvent(event) {
        var propertyKeys = Object.keys(event);
        var tableRow = document.querySelector("[event=" + event['code'] + "]");
        //iterate event attributes
        propertyKeys.forEach(function(key) {
            if(!Array.isArray(event[key])) {
                updateValue(tableRow, key, event[key])
            } else {
                //has children => additional properties like subscriptions
                var additionalProperties = event[key];
                var tableElement = document.querySelector("[additional-properties-code=" + key + "]");
                for(const property of additionalProperties) {
                    var propertyElement = tableElement.querySelector("[subscription-id=\"" + property.subscriptionId + "\"]");
                    if (propertyElement) {
                        updateSubscriptions(propertyElement, property)
                    }
                }
            }
        })
    }

    function updateServices(companyServices) {
        for (var service of companyServices) {
            var tableRow = document.querySelector("[service-code=" + service['serviceCode'] + "]");
            var serviceAttributes = Object.keys(service);

            //iterate service attributes and event
            serviceAttributes.forEach(function(key) {
                if (!Array.isArray(service[key])) {
                    // single attributes
                    updateValue(tableRow, key, service[key]);
                } else {
                    // event founded
                    var eventsWithAdditionalProperties = service[key];
                    // iterate events
                    for (var event of eventsWithAdditionalProperties) {
                        updateEvent(event);
                    }
                }
            })
        }
    }

    /**
     * Event handler callback
     * Continue in transaction if the semaphore has been released
     */
    function onMessage(event, webSocket) {
        var message = event.data;

        var currentWindow = windowUtils.getCurrentWindow();
        console.log(message);
        switch (message) {
            case RELEASE_MESSAGE:
                // release module
                (currentWindow.find('#okButton')).click();
                webSocket.close();
                break;
            case PING_MESSAGE:
                // send pong message back
                webSocket.send(PONG_MESSAGE);
                break;
            default:
                var statusContainer = JSON.parse(message);
                var companyServices = statusContainer['companyServices'];;
                updateServices(companyServices);
                break;
        }
    }



    ns.closeConnection = function() {

        socket.webSocket.close();
    }

    /** ---- public methods ---- */
    return {
        initialize: ns.initialize
    };
});