package cz.aimtec.dci.coregate.controller.coreservices;

import cz.aimtec.dci.coregate.api.controller.coreservices.CoreServicesApi;
import cz.aimtec.dci.coregate.api.model.coreservices.*;
import cz.aimtec.dci.coregate.service.ServicesProvider;
import cz.aimtec.dci.microservice.controller.AsynchronousMethod;
import cz.aimtec.dci.microservice.controller.CommonController;
import cz.aimtec.dci.microservice.controller.ServiceVersion;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.List;


/**
 * Core services controller.
 *
 * @author	hors
 * @version	7. 11. 2017/hors - created<br>
 */
@RestController
@ServiceVersion
public class ServiceController implements CommonController, CoreServicesApi {

	private final Logger		logger = LoggerFactory.getLogger(this.getClass());
	
	@Autowired
	private ServicesProvider	servicesProvider;

	@Autowired
	private HttpServletResponse response;
	
	
	@Override
	public ResponseEntity<List<ServiceConfiguration>> coreServicesConfigurationGet() throws Exception {
		HttpStatus responseStatus = HttpStatus.OK;
		List<ServiceConfiguration> services = null;

		// ----------------------- getting services configuration from dciCore
		services = servicesProvider.getConfiguration();
		responseStatus = services != null ? HttpStatus.OK : HttpStatus.NOT_FOUND;

		// ----------------------- return
		return new ResponseEntity<>(services, responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesExecutePost(ServiceExecution pExecutionParameters) {
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- service execution
		boolean isExecuted = servicesProvider.execute(pExecutionParameters.getServiceCode(),
													  pExecutionParameters.getCompanyId(),
													  pExecutionParameters.getEventCode());
		responseStatus = isExecuted ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesRegisterLicensePost(ServiceExecution pExecutionParameters) {
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- service license registration
		boolean isRegisterred = servicesProvider.registerLicence(pExecutionParameters.getServiceCode(),
																 pExecutionParameters.getCompanyId(),
																 pExecutionParameters.getEventCode());
		responseStatus = isRegisterred ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesDeregisterLicensePost(ServiceExecution pExecutionParameters) {
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- service license registration
		boolean isRegisterred = servicesProvider.deregisterLicence(pExecutionParameters.getServiceCode(),
																   pExecutionParameters.getCompanyId(),
																   pExecutionParameters.getEventCode());
		responseStatus = isRegisterred ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesInitializeMachinePost(ServiceExecution pExecutionParameters) {
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- machine initialization
		boolean isInitialized = servicesProvider.initializeMachine(pExecutionParameters.getServiceCode(),
																   pExecutionParameters.getCompanyId(),
																   pExecutionParameters.getEventCode());
		responseStatus = isInitialized ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesHandleMessagePost(cz.aimtec.dci.coregate.api.model.coreservices.IncomingMessage pMessage)
			throws Exception {
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- service license registration
		boolean isHandled = servicesProvider.handleMessage(pMessage);
		responseStatus = isHandled ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesArchiveMessagesPost(List<IncomingMessage> pMessages) {
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- service license registration
		boolean isArchived = servicesProvider.archiveMessages(pMessages);
		responseStatus = isArchived ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesSaveWatchdogLogsPost(String pServiceCode,
																 Integer pCompanyId,
																 List<WatchdogLog> pWatchdogLogs) throws Exception {
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- watchdog logs saving
		boolean isSaved = servicesProvider.saveWatchdogLogs(pServiceCode, pCompanyId, pWatchdogLogs);
		responseStatus = isSaved ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesSaveRfcServerStatePost(String pServiceCode,
																   Integer pCompanyId,
																   SAPRfcServerState pServerState) throws Exception {
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- RFC server state saving
		boolean isSaved = servicesProvider.saveRfcServerState(pServiceCode, pCompanyId, pServerState);
		responseStatus = isSaved ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	public ResponseEntity<SAPRfcTidStatus> coreServicesCheckRfcTidGet(String pServiceCode,
																	  Integer pCompanyId) throws Exception {
		HttpStatus responseStatus = HttpStatus.OK;
		SAPRfcTidStatus status = null;

		// ----------------------- check RFC TID
		status = servicesProvider.checkRfcTid(pServiceCode, pCompanyId);

		// ----------------------- return
		return new ResponseEntity<>(status, responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesHandleRfcMessagePost(String pServiceCode,
																 Integer pCompanyId,
																 SAPRfcMessage pMessage) throws Exception {
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- RFC server state saving
		boolean isHandled = servicesProvider.handleRfcMessage(pServiceCode, pCompanyId, pMessage);
		responseStatus = isHandled ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesSaveRfcServerErrorPost(String pServiceCode,
																   Integer pCompanyId,
																   SAPRfcServerError pServerError) throws Exception  {
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- RFC server state saving
		boolean isSaved = servicesProvider.saveRfcServerError(pServiceCode, pCompanyId, pServerError);
		responseStatus = isSaved ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	public ResponseEntity<Void> coreServicesSaveMachineResponseDurationPost(String pServiceCode,
																			Integer pCompanyId,
																			String pEventCode,
																			MachineResponseDuration pResponseDuration)
			throws Exception{
		HttpStatus responseStatus = HttpStatus.OK;

		// ----------------------- RFC server state saving
		boolean isSaved = servicesProvider.saveMachineResponseDuration(pServiceCode, pCompanyId, pEventCode, pResponseDuration);
		responseStatus = isSaved ? HttpStatus.OK : HttpStatus.NOT_MODIFIED;

		// ----------------------- return
		return new ResponseEntity<>(responseStatus);
	}

	@Override
	@AsynchronousMethod()
	public ResponseEntity<Void> coreServicesMonitorSystemServicesPost(@Valid StatusContainer statusContainer) throws Exception {
		boolean responseStatus = servicesProvider.monitorSystemServicesStatus(statusContainer);
		return new ResponseEntity<>(responseStatus ? HttpStatus.OK : HttpStatus.NOT_MODIFIED);
	}
}
