package cz.aimtec.dci.coreservices.listener.tcpip.opc;


import com.prosysopc.ua.ServiceException;
import com.prosysopc.ua.client.ServerStatusListener;
import com.prosysopc.ua.client.UaClient;
import com.prosysopc.ua.stack.builtintypes.LocalizedText;
import com.prosysopc.ua.stack.builtintypes.StatusCode;
import com.prosysopc.ua.stack.core.ServerState;
import com.prosysopc.ua.stack.core.ServerStatusDataType;
import cz.aimtec.dci.coreservices.listener.tcpip.TCPIPClient;
import cz.aimtec.util.Convert;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Default OPC server status listener. Handle status message actions.
 * @author kvaj
 *
 */
public class OPCServerStatusListener implements ServerStatusListener {
	
	private static final Logger	log = LoggerFactory.getLogger(OPCServerStatusListener.class);
	
	private OPCClient			opcClient = null;
	
	
	public OPCServerStatusListener(OPCClient opcClient) {
		super();
		this.opcClient = opcClient;
	}
	
	
	@Override
	public void onShutdown(UaClient uaClient,
	                       long secondsTillShutdown,
	                       LocalizedText shutdownReason) {
		TCPIPClient.registerListenerCodeForLogging(opcClient);
		log.info("Shutdown occured for OPC.");
		
		try {
			log.debug("OPCServerStatusListener.onShutdown ... disconnect");
			if (uaClient != null) {
				log.debug("OPCServerStatusListener.onShutdown ... disconnect");
				uaClient.disconnect();
			}
		} catch (Exception e) {
			log.debug("OPC disconnect failed. It is probably already disconnected (this is not an error, just a debug information).", e);
		}
	}

	@Override
	public void onStateChange(UaClient uaClient,
	                          ServerState oldState,
	                          ServerState newState) {
		TCPIPClient.registerListenerCodeForLogging(opcClient);

		if (log.isInfoEnabled()) {
			log.info("Server state changed from {} to {}!",
					Convert.toString(oldState), Convert.toString(newState));
		}

		if (!ServerState.Running.equals(oldState) && !ServerState.Test.equals(oldState) &&
				(ServerState.Running.equals(newState) || !ServerState.Test.equals(newState))) {
			try {
				opcClient.unsubscribeAll();
				opcClient.registerAllSubscriptions();
				opcClient.subscribeAll();
			} catch (ServiceException e) {
				log.error("Error during creating OPC subscriptions.", e);
			}
		}
	}

	@Override
	public void onStatusChange(UaClient uaClient, ServerStatusDataType status, StatusCode statusCode) {
		// Called whenever the server status changes, typically every
		// StatusCheckInterval defined in the UaClient.
	}
}
