package cz.aimtec.dci.coreservices.listener.tcpip.opc;

import com.prosysopc.ua.stack.builtintypes.StatusCode;
import cz.aimtec.dci.coreservices.api.model.OPCSubscriptionItemStatus;
import cz.aimtec.util.datatype.DateTime;


/**
 * Subscription item for TCPIPListener subscriptions.
 * In this version it is primarily designed for OPC subscriptions because other TCPIP listeners don't support subscriptions.
 * @author asel, koup
 * 
 */
public class SubscriptionItem {
	private Integer subscriptionId;
	private String nodeId;
	private String attributeId;
	private String transactionCode;
	private DateTime lastRunTime;
	private String publishingInterval;
	private boolean subscribed;


	/**
	 * Default constructor.
	 * @param nodeId
	 * @param attributeId
	 * @param transactionCode
	 * @param status
	 */
	public SubscriptionItem(Integer subscriptionId, String nodeId, String attributeId, String transactionCode, DateTime lastRunTime, String publishingInterval, String status) {
		this.subscriptionId = subscriptionId;
		this.nodeId = nodeId;
		this.attributeId = attributeId;
		this.transactionCode = transactionCode;
		this.lastRunTime = lastRunTime;
		this.publishingInterval = publishingInterval;
		if (StatusCode.GOOD.toString().equals(status)) {
			this.subscribed = true;
		} else {
			this.subscribed = false;
		}
	}

	public String getNodeId() {
		return nodeId;
	}
	
	public String getNodeIdShort() {
		return (nodeId.length() <= 20)
					? nodeId
					: nodeId.substring(0, 17) + "...";
	}

	public String getAttributeId() {
		return attributeId;
	}

	public DateTime getLastRunTime() {
		return lastRunTime;
	}

	public String getTransactionCode() {
		return transactionCode;
	}
	
	public String getPublishingInterval() {
		return publishingInterval;
	}
	
	
	public boolean isSubscribed() {
		return subscribed;
	}

	public Integer getSubscriptionId() {
		return subscriptionId;
	}

	public void setSubscriptionId(int subscriptionId) {
		this.subscriptionId = subscriptionId;
	}

	public void setSubscribed(String subscriptionStatus) {
		if (StatusCode.GOOD.toString().equals(subscriptionStatus)) {
			this.subscribed = true;
		} else {
			this.subscribed = false;
		}
	}

	public OPCSubscriptionItemStatus createSubscriptionItemStatus() {
		OPCSubscriptionItemStatus status = new OPCSubscriptionItemStatus();

		// ------------------------ item status
		status.setSubscriptionId(getSubscriptionId());
		status.setNodeId(getNodeId());
		status.setAttributeId(getAttributeId());
		status.setTransactionCode(getTransactionCode());
		status.setLastRunTime(getLastRunTime() != null ? getLastRunTime().convertToOffsetDateTime() : null);
		status.setPublishingInterval(getPublishingInterval());
		status.setSubscribed(isSubscribed());

		// ------------------------ return
		return status;
	}
}
