package cz.aimtec.dci.coreservices.implementation;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import cz.aimtec.dci.coreservices.api.model.ServiceConfiguration;
import cz.aimtec.dci.coreservices.api.model.StatusContainer;
import cz.aimtec.dci.coreservices.service.Service;
import cz.aimtec.dci.coreservices.service.ServiceScheduleType;
import cz.aimtec.dci.coreservices.service.ServicesDispatcher;
import cz.aimtec.dci.microservices.CoreGate;
import cz.aimtec.util.datatype.DateTime;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;

public class SystemServicesMonitor extends Service {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    private final int ADMIN_COMPANY_ID = 1;
    private ObjectMapper jsonMapper;
    private StatusContainer lastStatusContainer;

    @Autowired
    private CoreGate coreGate;

    @Autowired
    private ServicesDispatcher dispatcher;

    public SystemServicesMonitor(String pCode, Integer pCompanyId, long pInterval, ServiceScheduleType pScheduleType, DateTime pFirstTime, int pCheckInterval, boolean pEnabled) {
        super(pCode, pCompanyId, pInterval, pScheduleType, pFirstTime, pCheckInterval, pEnabled);
    }

    public SystemServicesMonitor(String pCode, Integer pCompanyId) {
        super(pCode, pCompanyId);
    }

    public SystemServicesMonitor(ServiceConfiguration pConfiguration) {
        super(pConfiguration);
    }

    @Override
    public void run() {
        logger.debug("Seding message to dciCore via EventBus.");
        StatusContainer actualStatusContainer = getStatusContainerFromServicesSet();
        try {
            if (jsonMapper == null) {
                jsonMapper = new ObjectMapper();
                jsonMapper.registerModule(new JavaTimeModule());
            }
            if (lastStatusContainer == null) {
                lastStatusContainer = actualStatusContainer;
            } else {
                actualStatusContainer.getCompanyServices().removeIf(s -> lastStatusContainer.getCompanyServices().contains(s));
                lastStatusContainer = actualStatusContainer;
            }
            if (!actualStatusContainer.getCompanyServices().isEmpty() && !actualStatusContainer.getApplicationServices().isEmpty()) {
                cz.aimtec.dci.coregate.api.model.coreservices.StatusContainer s = jsonMapper.readValue(jsonMapper.writeValueAsString(actualStatusContainer), cz.aimtec.dci.coregate.api.model.coreservices.StatusContainer.class);
                coreGate.sendServicesStateToDCI(s);
            }
        } catch (Exception e) {
            logger.error("Error while sending message to dciCore.", e);
        }
    }


    private StatusContainer getStatusContainerFromServicesSet() {
        StatusContainer servicesStatus = new StatusContainer();
        servicesStatus.setCompanyServices(new ArrayList<>());
        servicesStatus.setApplicationServices(new ArrayList<>());

        try {
            // ----------------------- active company services
            // TODO – now it is only for company AIMTEC
            for (Service service : dispatcher.getActiveServices(ADMIN_COMPANY_ID)) {
                servicesStatus.getCompanyServices().add(service.createServiceStatus());
            }

            // ----------------------- active application services
            for (Service service : dispatcher.getActiveServices()) {
                servicesStatus.getApplicationServices().add(service.createServiceStatus());
            }

        } catch (Exception exception) {
            logger.error("Error while retrieving status container from dispatcher.", exception);
        }
        return servicesStatus;
    }

    @Override
    public synchronized void activate() {
        super.activate();
        lastStatusContainer = getStatusContainerFromServicesSet();
    }
}
